@@ -1,12 +1,38 @@
 """Just-in-time Python extension compilation."""
 
 from __future__ import division
+
+__copyright__ = """
+Copyright (C) 2009-17 Andreas Kloeckner
+Copyright (C) 2017 Nick Curtis
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
 from codepy import CompileError
 from pytools import Record
+import six
 
-__copyright__ = "Copyright (C) 2008 Andreas Kloeckner"
-
-
+import logging
+logger = logging.getLogger(__name__)
 
 
 def _erase_dir(dir):
@@ -17,8 +43,6 @@ def _erase_dir(dir):
     rmdir(dir)
 
 
-
-
 def extension_file_from_string(toolchain, ext_file, source_string,
         source_name="module.cpp", debug=False):
     """Using *toolchain*, build the extension file named *ext_file*
@@ -43,11 +67,10 @@ def extension_file_from_string(toolchain, ext_file, source_string,
         _erase_dir(src_dir)
 
 
-
-
 class CleanupBase(object):
     pass
 
+
 class CleanupManager(CleanupBase):
     def __init__(self):
         self.cleanups = []
@@ -63,6 +86,7 @@ class CleanupManager(CleanupBase):
         for c in self.cleanups:
             c.error_clean_up()
 
+
 class TempDirManager(CleanupBase):
     def __init__(self, cleanup_m):
         from tempfile import mkdtemp
@@ -79,6 +103,7 @@ class TempDirManager(CleanupBase):
     def error_clean_up(self):
         pass
 
+
 class CacheLockManager(CleanupBase):
     def __init__(self, cleanup_m, cache_dir):
         import os
@@ -115,6 +140,7 @@ class CacheLockManager(CleanupBase):
     def error_clean_up(self):
         pass
 
+
 class ModuleCacheDirManager(CleanupBase):
     def __init__(self, cleanup_m, path):
         from os import mkdir
@@ -178,32 +204,27 @@ def extension_from_string(toolchain, name, source_string,
                             source_name,
                             cache_dir, debug, wait_on_error, debug_recompile,
                             False)
-     # try loading it
+    # try loading it
     from imp import load_dynamic
     return load_dynamic(mod_name, ext_file)
 
 
-
-
 class _InvalidInfoFile(RuntimeError):
     pass
 
 
-
 class _SourceInfo(Record):
     pass
 
 
-
-
 def compile_from_string(toolchain, name, source_string,
-                        source_name="module.cpp", cache_dir=None,
+                        source_name=["module.cpp"], cache_dir=None,
                         debug=False, wait_on_error=None, debug_recompile=True,
-                        object=False):
+                        object=False, source_is_binary=False):
     """Returns a tuple: mod_name, file_name, recompiled.
     mod_name is the name of the module represented by a compiled object,
     file_name is the name of the compiled object, which can be built from the
-    source code in *source_string* if necessary,
+    source code(s) in *source_strings* if necessary,
     recompiled is True if the object had to be recompiled, False if the cache
     is hit.
     Raise :exc:`CompileError` in case of error.  The mod_name and file_name
@@ -232,8 +253,18 @@ def compile_from_string(toolchain, name, source_string,
     If *debug_recompile*, messages are printed indicating whether a
     recompilation is taking place.
 
+    If *source_is_binary*, the source string is a compile object file and
+    should be treated as binary for read/write purposes
     """
 
+    # first ensure that source strings and names are lists
+    if isinstance(source_string, six.string_types) \
+            or (source_is_binary and isinstance(source_string, six.binary_type)):
+        source_string = [source_string]
+
+    if isinstance(source_name, str):
+        source_name = [source_name]
+
     if wait_on_error is not None:
         from warnings import warn
         warn("wait_on_error is deprecated and has no effect",
@@ -243,14 +274,16 @@ def compile_from_string(toolchain, name, source_string,
     from os.path import join
 
     if cache_dir is None:
-        from os.path import exists
-        from tempfile import gettempdir
-        cache_dir = join(gettempdir(),
-                "codepy-compiler-cache-v5-uid%s" % os.getuid())
+        import appdirs
+        import sys
+        cache_dir = join(
+                appdirs.user_cache_dir("codepy", "codepy"),
+                "codepy-compiler-cache-v5-py%s" % (
+                    ".".join(str(i) for i in sys.version_info),))
 
         try:
-            os.mkdir(cache_dir)
-        except OSError, e:
+            os.makedirs(cache_dir)
+        except OSError as e:
             from errno import EEXIST
             if e.errno != EEXIST:
                 raise
@@ -264,21 +297,23 @@ def compile_from_string(toolchain, name, source_string,
             import md5
             checksum = md5.new()
 
-        inf = open(fname)
+        inf = open(fname, "rb")
         checksum.update(inf.read())
+
         inf.close()
         return checksum.hexdigest()
 
-    def get_dep_structure(source_path):
-        deps = list(toolchain.get_dependencies([source_path]))
+    def get_dep_structure(source_paths):
+        deps = list(toolchain.get_dependencies(source_paths))
         deps.sort()
         return [(dep, os.stat(dep).st_mtime, get_file_md5sum(dep)) for dep in deps
-                if not dep == source_path]
+                if dep not in source_paths]
 
     def write_source(name):
-        outf = open(name, "w")
-        outf.write(str(source_string))
-        outf.close()
+        for i, source in enumerate(source_string):
+            outf = open(name[i], "w" if not source_is_binary else "wb")
+            outf.write(source)
+            outf.close()
 
     def calculate_hex_checksum():
         try:
@@ -289,15 +324,19 @@ def compile_from_string(toolchain, name, source_string,
             import md5
             checksum = md5.new()
 
-        checksum.update(source_string)
-        checksum.update(str(toolchain.abi_id()))
+        for source in source_string:
+            if source_is_binary:
+                checksum.update(source)
+            else:
+                checksum.update(source.encode('utf-8'))
+        checksum.update(str(toolchain.abi_id()).encode('utf-8'))
         return checksum.hexdigest()
 
     def load_info(info_path):
-        from cPickle import load
+        from six.moves.cPickle import load
 
         try:
-            info_file = open(info_path)
+            info_file = open(info_path, 'rb')
         except IOError:
             raise _InvalidInfoFile()
 
@@ -308,46 +347,50 @@ def compile_from_string(toolchain, name, source_string,
         finally:
             info_file.close()
 
-
     def check_deps(deps):
         for name, date, md5sum in deps:
             try:
                 possibly_updated = os.stat(name).st_mtime != date
-            except OSError, e:
+            except OSError as e:
                 if debug_recompile:
-                    print("recompiling because dependency %s is "
+                    logger.info("recompiling because dependency %s is "
                     "inaccessible (%s)." % (name, e))
                 return False
             else:
                 if possibly_updated and md5sum != get_file_md5sum(name):
                     if debug_recompile:
-                        print("recompiling because dependency %s was "
+                        logger.info("recompiling because dependency %s was "
                         "updated." % name)
                     return False
 
         return True
 
     def check_source(source_path):
-        try:
-            src_f = open(source_path, "r")
-        except IOError:
-            if debug_recompile:
-                print ("recompiling because cache directory does "
-                        "not contain source file '%s'." % source_path)
-            return False
+        valid = True
+        for i, path in enumerate(source_path):
+            source = source_string[i]
+            try:
+                src_f = open(path, "r" if not source_is_binary else "rb")
+            except IOError:
+                if debug_recompile:
+                    logger.info("recompiling because cache directory does "
+                            "not contain source file '%s'." % path)
+                return False
 
-        valid = src_f.read() == source_string
-        src_f.close()
+            valid = valid and src_f.read() == source
+            src_f.close()
 
-        if not valid:
-            from warnings import warn
-            warn("hash collision in compiler cache")
+            if not valid:
+                from warnings import warn
+                warn("hash collision in compiler cache")
         return valid
 
     cleanup_m = CleanupManager()
 
     try:
-        lock_m = CacheLockManager(cleanup_m, cache_dir)
+        # Variable 'lock_m' is used for no other purpose than
+        # to keep lock manager alive.
+        lock_m = CacheLockManager(cleanup_m, cache_dir)  # noqa
 
         hex_checksum = calculate_hex_checksum()
         mod_name = "codepy.temp.%s.%s" % (hex_checksum, name)
@@ -355,51 +398,45 @@ def compile_from_string(toolchain, name, source_string,
             suffix = toolchain.o_ext
         else:
             suffix = toolchain.so_ext
-        if cache_dir:
-            mod_cache_dir_m = ModuleCacheDirManager(cleanup_m,
-                    join(cache_dir, hex_checksum))
-            info_path = mod_cache_dir_m.sub("info")
-            ext_file = mod_cache_dir_m.sub(name+suffix)
 
-            if mod_cache_dir_m.existed:
-                try:
-                    info = load_info(info_path)
-                except _InvalidInfoFile:
-                    mod_cache_dir_m.reset()
+        mod_cache_dir_m = ModuleCacheDirManager(cleanup_m,
+                join(cache_dir, hex_checksum))
+        info_path = mod_cache_dir_m.sub("info")
+        ext_file = mod_cache_dir_m.sub(name+suffix)
+
+        if mod_cache_dir_m.existed:
+            try:
+                info = load_info(info_path)
+            except _InvalidInfoFile:
+                mod_cache_dir_m.reset()
 
-                    if debug_recompile:
-                        print "recompiling for invalid cache dir (%s)." % (
-                                mod_cache_dir_m.path)
-                else:
-                    if check_deps(info.dependencies) and check_source(
-                            mod_cache_dir_m.sub(info.source_name)):
-                        return hex_checksum, mod_name, ext_file, False
-            else:
                 if debug_recompile:
-                    print "recompiling for non-existent cache dir (%s)." % (
-                            mod_cache_dir_m.path)
-
-            source_path = mod_cache_dir_m.sub(source_name)
+                    logger.info("recompiling for invalid cache dir (%s)." % (
+                            mod_cache_dir_m.path))
+            else:
+                if check_deps(info.dependencies) and check_source(
+                        [mod_cache_dir_m.sub(x) for x in info.source_name]):
+                    return hex_checksum, mod_name, ext_file, False
         else:
-            ext_file = join(temp_dir, source_name+suffix)
-            done_path = None
-            info_path = None
+            if debug_recompile:
+                logger.info("recompiling for non-existent cache dir (%s)." % (
+                        mod_cache_dir_m.path))
 
-            temp_dir_m = TempDirManager(cleanup_m)
-            source_path = temp_dir_m.sub(source_name)
+        source_paths = [mod_cache_dir_m.sub(source) for source in source_name]
 
-        write_source(source_path)
+        write_source(source_paths)
 
         if object:
-            toolchain.build_object(ext_file, [source_path], debug=debug)
+            toolchain.build_object(ext_file, source_paths, debug=debug)
         else:
-            toolchain.build_extension(ext_file, [source_path], debug=debug)
+            toolchain.build_extension(ext_file, source_paths, debug=debug)
 
         if info_path is not None:
-            from cPickle import dump
-            info_file = open(info_path, "w")
+            from six.moves.cPickle import dump
+
+            info_file = open(info_path, "wb")
             dump(_SourceInfo(
-                dependencies=get_dep_structure(source_path),
+                dependencies=get_dep_structure(source_paths),
                 source_name=source_name), info_file)
             info_file.close()
 
@@ -411,8 +448,6 @@ def compile_from_string(toolchain, name, source_string,
         cleanup_m.clean_up()
 
 
-
-
 def link_extension(toolchain, objects, mod_name, cache_dir=None,
         debug=False, wait_on_error=True):
     import os.path
@@ -421,13 +456,14 @@ def link_extension(toolchain, objects, mod_name, cache_dir=None,
     else:
         # put the linked object in the same directory as the first object
         destination_base, first_object = os.path.split(objects[0])
-        destination = os.path.join(destination_base, mod_name
-                                   + toolchain.so_ext)
+        destination = os.path.join(
+                destination_base,
+                mod_name + toolchain.so_ext)
     try:
         toolchain.link_extension(destination, objects, debug=debug)
     except CompileError:
         if wait_on_error:
-            raw_input("Link error, examine %s, then press [Enter]" % objects)
+            six.moves.input("Link error, examine %s, then press [Enter]" % objects)
             raise
 
     # try loading it
@@ -435,9 +471,7 @@ def link_extension(toolchain, objects, mod_name, cache_dir=None,
     return load_dynamic(mod_name, destination)
 
 
+from pytools import MovedFunctionDeprecationWrapper  # noqa: E402
+from codepy.toolchain import guess_toolchain as _gtc  # noqa: E402
 
-
-from pytools import MovedFunctionDeprecationWrapper
-from codepy.toolchain import guess_toolchain as _gtc
 guess_toolchain = MovedFunctionDeprecationWrapper(_gtc)
-
